import { __assign } from 'tslib';
import { useRef, useEffect } from 'react';
import { getValueTransition } from '../../../animation/utils/transitions.js';
import { createBox } from '../../../projection/geometry/models.js';
import { boxEquals } from '../../../projection/geometry/utils.js';
import { animateDelta } from '../../../projection/animation/index.js';

function AnimateLayout(_a) {
    var visualElement = _a.visualElement, layout = _a.layout, layoutId = _a.layoutId;
    var projection = visualElement.projection;
    var layoutTarget = useRef(createBox());
    useEffect(function () {
        if (!(layout || layoutId) || !projection)
            return;
        return projection.onLayoutDidUpdate(function (_a) {
            var newLayout = _a.layout, delta = _a.delta, hasLayoutChanged = _a.hasLayoutChanged;
            // TODO: Check here if an animation exists
            var layoutTransition = visualElement.getDefaultTransition() ||
                defaultLayoutTransition;
            var onLayoutAnimationComplete = visualElement.getProps().onLayoutAnimationComplete;
            if (hasLayoutChanged &&
                /**
                 * Don't create a new animation if the target box
                 * hasn't changed TODO: And we're already animating
                 */
                !boxEquals(layoutTarget.current, newLayout)) {
                animateDelta(projection, delta, __assign(__assign({}, getValueTransition(layoutTransition, "layout")), { onComplete: onLayoutAnimationComplete }));
            }
            layoutTarget.current = newLayout;
        });
    }, [layout, layoutId]);
    return null;
}
var defaultLayoutTransition = {
    duration: 0.45,
    ease: [0.4, 0, 0.1, 1],
};

export { AnimateLayout };
